import hashlib
import os
import sys
from typing import BinaryIO

DEFAULT_CHUNK_SIZE_MB = 64


def sha3_384_hash(data: bytes) -> bytes:
    hasher = hashlib.sha3_384()
    hasher.update(data)
    return hasher.digest()


# Remark: Thanks to the use of ‘hasher.update()’, we do not need to load the entire chunk into memory.
def streamed_chunk_hash(f: BinaryIO, chunk_size_mb: int) -> bytes or None:
    hasher = hashlib.sha3_384()

    consumed_mb = 0
    while consumed_mb < chunk_size_mb:
        data_mb = f.read(1024 * 1024)
        if not data_mb:
            break
        hasher.update(data_mb)
        consumed_mb += 1

    if consumed_mb > 0:
        return hasher.digest()
    else:
        return None


def calculate_chunked_hash(filename: str, chunk_size_mb: int) -> bytes:
    if not os.path.isfile(filename):
        print(f"Error: File '{filename}' not found.")
        sys.exit(1)

    overall_hash = sha3_384_hash(bytes(0))  # for the case of empty input

    with open(filename, 'rb') as f:
        chunk_num = 0
        while chunk_hash := streamed_chunk_hash(f, chunk_size_mb):

            if chunk_num == 0:
                overall_hash = chunk_hash
            else:
                overall_hash = sha3_384_hash(overall_hash + chunk_hash)

            chunk_num += 1

    return overall_hash


if __name__ == "__main__":
    if len(sys.argv) < 2 or len(sys.argv) >= 3 and not sys.argv[2].isnumeric():
        print(f"Usage: python {sys.argv[0]} <filename> [chunk_size_MB]")
        print(f"Default chunk size: {DEFAULT_CHUNK_SIZE_MB} MB")
        sys.exit(1)

    input_file = sys.argv[1]
    effective_chunk_size_mb = int(sys.argv[2]) if len(sys.argv) > 2 else DEFAULT_CHUNK_SIZE_MB
    result = calculate_chunked_hash(input_file, effective_chunk_size_mb)

    print(result.hex())
